<?php
namespace Teknisa\CTB\Factories;

use Doctrine\ORM\EntityManager;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;

class Ctb32700DREAnualFactory {

    protected $connection;

    /**
     * @param EntityManager $entityManager
     */
    public function __construct(EntityManager $entityManager) {
        $this->connection = $entityManager->getConnection();
        $this->entityManager = $entityManager;
    }
    
    /**
     * Busca Tmpcontabger
     * @param $nrcontabger
     * @return Tmpcontabger
     */
    public function buscaTmpcontabger($nrcontabger){
        /** @var $params */
        $params = array();
        $params['P_NRCONTABGER'] = $nrcontabger;

        $sql = "  SELECT CDFILIALTMP, CDTIPODEMOTMP, CDITEMSTRUTMP, CDCONTCTBLTMP,
                     TO_CHAR(DTREFETMP01,'MM/YYYY') AS DTREFETMP01,
                     TO_CHAR(DTREFETMP02,'MM/YYYY') AS DTREFETMP02,
                     TO_CHAR(DTREFETMP03,'MM/YYYY') AS DTREFETMP03,
                     TO_CHAR(DTREFETMP04,'MM/YYYY') AS DTREFETMP04,
                     TO_CHAR(DTREFETMP05,'MM/YYYY') AS DTREFETMP05,
                     TO_CHAR(DTREFETMP06,'MM/YYYY') AS DTREFETMP06,
                     TO_CHAR(DTREFETMP07,'MM/YYYY') AS DTREFETMP07,
                     TO_CHAR(DTREFETMP08,'MM/YYYY') AS DTREFETMP08,
                     TO_CHAR(DTREFETMP09,'MM/YYYY') AS DTREFETMP09,
                     TO_CHAR(DTREFETMP10,'MM/YYYY') AS DTREFETMP10,
                     TO_CHAR(DTREFETMP11,'MM/YYYY') AS DTREFETMP11,
                     TO_CHAR(DTREFETMP12,'MM/YYYY') AS DTREFETMP12,
                     VRREFETMP01, VRREFETMP02, VRREFETMP03, VRREFETMP04, VRREFETMP05, VRREFETMP06,
                     VRREFETMP07, VRREFETMP08, VRREFETMP09, VRREFETMP10, VRREFETMP11, VRREFETMP12,
                     F.NMFILIAL, D.NMTIPODEMO, I.NMITEMSTRU, C.NMCONTCTBL, I.IDOPERACAO
                FROM TMPCONTABGER T, FILIAL F, TIPODEMO D, ITEMDEMOF I, CONTCTBL C
               WHERE T.NRCONTABGER   = :P_NRCONTABGER
                 AND T.CDFILIALTMP   = F.CDFILIAL
                 AND T.CDTIPODEMOTMP = D.CDTIPODEMO
                 AND T.CDTIPODEMOTMP = I.CDTIPODEMO
                 AND T.CDITEMSTRUTMP = I.CDITEMSTRU
                 AND T.CDCONTCTBLTMP = C.CDCONTCTBL(+)
                 AND ((VRREFETMP01 <> 0) OR (VRREFETMP02 <> 0) OR (VRREFETMP03 <> 0) OR (VRREFETMP04 <> 0) OR
                      (VRREFETMP05 <> 0) OR (VRREFETMP06 <> 0) OR (VRREFETMP07 <> 0) OR (VRREFETMP08 <> 0) OR
                      (VRREFETMP09 <> 0) OR (VRREFETMP10 <> 0) OR (VRREFETMP11 <> 0) OR (VRREFETMP12 <> 0))
            ORDER BY CDFILIALTMP, CDTIPODEMOTMP, CDITEMSTRUTMP, CDCONTCTBLTMP";

        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    /**
     * Insere Lista
     * @param $nrlista
     * @param $cdlista
     */
    public function insereLista($nrlista, $cdlista){
        /** @var $params */
        $params = array();
        $params['P_NRLISTA'] = $nrlista;
        $params['P_CDLISTA'] = $cdlista;

        $sql = "INSERT INTO LISTA
                    (NRLISTA, CDLISTA)
                    (SELECT :P_NRLISTA, CDFILIAL FROM FILIAL WHERE INSTR(:P_CDLISTA, CDFILIAL) > 0)";

        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }

    /**
     * Executa Procedure GERA_DREGERENCIAL
     * @param $cdempresa
     * @param $nrlista
     * @param $cdtipodemo
     * @param $dtinicial
     * @param $dtfinal
     * @param $nrcontador
     * @param $vrtxadmcent
     * @return mixed
     */
    public function geraDREAnual(
        $cdempresa, $nrlista, $cdtipodemo, $dtinicial, $dtfinal, $nrcontador, $vrtxadmcent
    ){
        $procedure = new StoredProcedure($this->entityManager->getConnection(), 'GERA_DREGERENCIAL');
        $procedure->addParam(new Param('P_CDEMPRESA'  , Param::PARAM_INPUT , $cdempresa   , Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRLISTA'    , Param::PARAM_INPUT , $nrlista     , Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDTIPODEMO' , Param::PARAM_INPUT , $cdtipodemo  , Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DTINICIAL'  , Param::PARAM_INPUT , $dtinicial   , Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DTFINAL'    , Param::PARAM_INPUT , $dtfinal     , Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRCONTADOR' , Param::PARAM_INPUT , $nrcontador  , Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_VRTXADMCENT', Param::PARAM_INPUT , $vrtxadmcent , Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRCONTABGER', Param::PARAM_OUTPUT, null         , Param::PARAM_TYPE_STR, 12));
        $procedure = $procedure->execute();

        return $procedure['P_NRCONTABGER'];
    }

}